


#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "audio_basics.h"





extern const float exp_tab    [];
extern const float log_tab    [];
extern const float env_times  [];
extern const float note_phis  [];
extern const float sintab     [];
extern const float k_table    [];
extern const float q_table    [];
extern const float db_table   [];

//*******************************************************************************
//*
//*
//*
//*******************************************************************************

void ENV_Init (ENV_AHDSR *env)
{
  memset (env, 0, sizeof (ENV_AHDSR));
}

void ENV_SetCalc (ENV_AHDSR *env)
{
  if (env->env_data.at > 127) { env->env_data.at = 127; }
  if (env->env_data.al > 127) { env->env_data.al = 127; }
  if (env->env_data.dt > 127) { env->env_data.dt = 127; }
  if (env->env_data.sl > 127) { env->env_data.sl = 127; }
  if (env->env_data.rt > 127) { env->env_data.rt = 127; }
  if (env->env_data.ac > 63)  { env->env_data.ac = 63;  }
  if (env->env_data.dc > 63)  { env->env_data.dc = 63;  }
  if (env->env_data.rc > 63)  { env->env_data.rc = 63;  }
  if (env->env_data.ac <-64)  { env->env_data.ac =-64;  }
  if (env->env_data.dc <-64)  { env->env_data.dc =-64;  }
  if (env->env_data.rc <-64)  { env->env_data.rc =-64;  }

  env->att_phiinc = (float) env->env_data.at / 128.0f;
  env->att_level  = (float) env->env_data.al / 127.0f;
  env->att_curve  = (float) env->env_data.ac /  64.0f;
  env->dec_phiinc = (float) env->env_data.dt / 128.0f;
  env->dec_curve  = (float) env->env_data.dc /  64.0f;
  env->sus_level  = (float) env->env_data.sl / 127.0f;
  env->rel_phiinc = (float) env->env_data.rt / 128.0f;
  env->rel_curve  = (float) env->env_data.rc /  64.0f;  

  env->att_phiinc = env_times [env->env_data.at];
  env->dec_phiinc = env_times [env->env_data.dt];
  env->rel_phiinc = env_times [env->env_data.rt];
  if (env->env_data.at == 0) { env->att_phiinc = 1.0f; }
  if (env->env_data.dt == 0) { env->dec_phiinc = 1.0f; }
  if (env->env_data.rt == 0) { env->rel_phiinc = 1.0f; }
}

void ENV_SetItem (ENV_AHDSR *env, uint8_t item, uint8_t val)
{
  switch (item)
  {
    case 1 : env->env_data.at = ((uint8_t) val > 127) ? 127 : (uint8_t) val; break;
    case 2 : env->env_data.ac = ((int8_t)  val > 63)  ? 63  : (int8_t)  val;
             env->env_data.ac = ((int8_t)  val <-64)  ?-64  : (int8_t)  val; break;
    case 3 : env->env_data.al = ((uint8_t) val > 127) ? 127 : (uint8_t) val; break;
    case 4 : env->env_data.dt = ((uint8_t) val > 127) ? 127 : (uint8_t) val; break;
    case 5 : env->env_data.dc = ((int8_t)  val > 63)  ? 63  : (int8_t)  val;
             env->env_data.dc = ((int8_t)  val <-64)  ?-64  : (int8_t)  val; break;
    case 6 : env->env_data.sl = ((uint8_t) val > 127) ? 127 : (uint8_t) val; break;
    case 7 : env->env_data.rt = ((uint8_t) val > 127) ? 127 : (uint8_t) val; break;
    case 8 : env->env_data.rc = ((int8_t)  val > 63)  ? 63  : (int8_t)  val;
             env->env_data.rc = ((int8_t)  val <-64)  ?-64  : (int8_t)  val; break;
  }
  ENV_SetCalc (env);
}


void ENV_Set (ENV_AHDSR *env, uint8_t at, uint8_t al, int8_t ac, uint8_t dt, int8_t dc, uint8_t sl, uint8_t rt, int8_t rc)
{
  env->env_data.at= at;
  env->env_data.al= al;
  env->env_data.dt= dt;
  env->env_data.sl= sl;
  env->env_data.rt= rt;
  env->env_data.ac= ac;
  env->env_data.dc= dc;
  env->env_data.rc= rc;

  ENV_SetCalc (env);
}

void ENV_SetStruct (ENV_AHDSR *env, ENV_AHDSR_DATA *data)
{
  env->env_data = *data;
  ENV_SetCalc (env);
}




float ENV_Step (ENV_AHDSR *env, uint8_t gate)
{
  uint16_t    phi_tidx;
  float       phi_out;

  switch (env->phase)
  {
    case 0 :
      if (gate) 
      {
        env->curr_gate    = ((float) gate) / 128;
        env->phase        = 1;  
        env->phase_phi    = 0.0f;
        env->phase_level  = 0.0f;
        env->phase_delta  = env->att_level;
        env->curr_curve   = env->att_curve;
        env->last_gate    = env->curr_gate * env->gate_int + (1.0f - env->gate_int);
      }
      break;

    case 1 :
      if (gate) 
      {
        env->phase_phi   +=  env->att_phiinc; 
        if (env->phase_phi >= 1.0f)  
        { 
          env->phase        = 2;
          env->phase_phi    = 0.0f; 
          env->phase_level  =  env->att_level; 
          env->phase_delta  = -(env->att_level - env->sus_level);
          env->curr_curve   =  env->dec_curve;
        }
      }
      else
      {
        env->phase        = 4;  
        env->phase_phi    =  0.0f; 
        env->phase_level  =  env->curr_level; 
        env->phase_delta  = -env->curr_level; 
        env->curr_curve   =  env->rel_curve;  
      }
      break;

    case 2 :
      if (gate) 
      {
        env->phase_phi   += env->dec_phiinc; 
        if (env->phase_phi >= 1.0f)                                               
        { 
          env->phase        = 3; 
          env->phase_phi    = 0.0f; 
          env->phase_level  = env->sus_level; 
          env->phase_delta  = 0.0f; 
          env->curr_curve   = 0.0f;
        }
      }
      else
      {
        env->phase        =  4;  
        env->phase_phi    =  0.0f; 
        env->phase_level  =  env->curr_level; 
        env->phase_delta  = -env->curr_level; 
        env->curr_curve   =  env->rel_curve;
      }
      break;

    case 3 :
      if (!gate) 
      {
        env->phase        =  4; 
        env->phase_phi    =  0.0f; 
        env->phase_level  =  env->sus_level; 
        env->phase_delta  = -env->sus_level; 
        env->curr_curve   =  env->rel_curve;
      }
      break;

    case 4 :
      if (!gate) 
      {
        env->phase_phi += env->rel_phiinc; 
        if (env->phase_phi >= 1.0f)
        {
          env->phase        = 5; 
          env->phase_phi    = 0.0f; 
          env->phase_level  = 0.0f;
          env->phase_delta  = 0.0f;
          env->curr_curve   = 0.0f;
        }   
      }
      else
      {
        env->phase        = 1;  
        env->phase_phi    = 0.0f;
        env->phase_level  = env->curr_level;
        env->phase_delta  = env->att_level - env->curr_level;
        env->curr_curve   = env->att_curve;
      }
      break;

    case 5 :
      if (!gate) 
      {
        env->phase = 0;  
      }
      break;
  }
  
  phi_tidx = ((uint16_t) (env->phase_phi * 1023.0f)) & 0x3ff;
  phi_out = (env->curr_curve >= 0) ? env->curr_curve * exp_tab [phi_tidx] + (1.0f -   env->curr_curve)  * env->phase_phi : 
                                    -env->curr_curve * log_tab [phi_tidx] + (1.0f - (-env->curr_curve)) * env->phase_phi ;

  env->curr_level     = (env->phase_level + env->phase_delta * phi_out) * env->last_gate;

  return env->curr_level;
}



//*******************************************************************************
//*
//*
//*
//*******************************************************************************


void OSC_SIN_Init (OSC_SIN *osc)
{
  memset (osc, 0, sizeof (OSC_SIN));
}

void OSC_SIN_SetNote (OSC_SIN *osc, float note)
{
  //uint32_t idx;

  osc->note = note > 119.99f ? 119.99f : note < 0.0f ? 0.0f : note;
/*
  idx = (uint16_t) (osc->note * 100.0f);
  osc->phiinc  = note_phis [idx % 1200] / (float) (1 << (9 - (idx / 1200)));*/

 osc->phiinc  = (440.0f * (float) pow (1.05946f, note - 69.0f)) / 48000.0f;
}

void OSC_SIN_SetAmp (OSC_SIN *osc, float amp)
{
  osc->amp = amp > 1.0f ? 1.0f : amp < 0.0f ? 0.0f : amp;
}

float OSC_SIN_CalcStep (OSC_SIN *osc)
{
  osc->phi += osc->phiinc;
  osc->phi -= (float) (uint16_t) osc->phi;
  return (sintab [((uint16_t) (osc->phi * 4096))]) * osc->amp;
}

void OSC_SIN_Calc (OSC_SIN *osc, float *out, int length)
{
  uint16_t  i;
  float     y;

  for (i = 0; i < length; i++)
  {
    osc->phi += osc->phiinc;
    osc->phi -= (float) (uint16_t) osc->phi;
    y = sintab [((uint16_t) (osc->phi * 4096))];
    out [i] = y * osc->amp;
  }
}




//*******************************************************************************
//*
//*
//*
//*******************************************************************************

/*    typedef struct
    {
      float note;
      float phi;
      float phiinc;  
      float amp;
    } OSC_SAW;


    void  OSC_SAW_Init      (OSC_SAW *osc);
    void  OSC_SAW_SetNote   (OSC_SAW *osc, float note);
    void  OSC_SAW_SetAmp    (OSC_SAW *osc, float amp);
    float OSC_SAW_CalcStep  (OSC_SAW *osc);
    void  OSC_SAW_Calc      (OSC_SAW *osc, float *out, int length);
*/


extern const float *saw_wavetable [3];

float test_table [4096];

void OSCSAWTableInit (void)
{
  float     coeffs [1024];
  float    *table, x, cs;
  uint16_t  i, j, k, jm, idx;
  float     p, p2, pi;

  for (i = 0; i < 1024; i++)
  {
    coeffs [i] = ((i & 1) == 1 ? -1.0f : 1.0f) * 1.0f / ((float) (i + 1));
  }

  for (k = 0; k < 3; k++)
  {
    pi = 55.000f * ((float) (1 << k)) / 44100.0f;
    table = (float *) malloc (4096 * sizeof (float));

    p2 = pi;
    cs = 0.0f;
    jm = 0;
    while (p2 < 0.5f)
    {
      p2 += pi;
      cs += (float) fabs (coeffs [jm]);
      jm++;
    }
    cs = 1 / cs;

    p = 0.0f; 
    pi = 1.0f / 4096.0f;
    for (i = 0; i < 4096; i++)
    {
      x   = 0.0f;
      p2  = p;
      for (j = 0; j < jm; j++)
      {
        idx = (uint16_t) (4095.0f * (p2 - ((float) (uint16_t) p2)));
        x += coeffs [j] * sintab [idx];
        p2 += p;
      }
      p  += pi;
      table [i] = x * cs;
      if (k == 0) { test_table [i] = table [i]; }
    }
    saw_wavetable [k] = table;
  }
}

void OSC_SAW_Init (OSC_SAW *osc)
{
  memset (osc, 0, sizeof (OSC_SAW));
}

void OSC_SAW_SetNote (OSC_SAW *osc, float note)
{
  uint32_t idx;

  osc->note = note > 56.99f ? 56.99f : note < 21.0f ? 21.0f : note;

  idx = ((uint16_t)  (osc->note * 100.0f));
  osc->tab = (uint8_t) ((osc->note - 21.0f) / 12.0f);
  if (osc->tab > 2) { osc->tab = 2; }
  osc->wavetable = (float  *) saw_wavetable [osc->tab];
  //osc->phiinc  = (note_phis [idx % 1200] / note_phis [0]) * (((float) (1 << (osc->tab))) / 4095.0f);
  osc->phiinc  = note_phis [idx % 1200] / (float) (1 << (9 - (idx / 1200)));
}

void OSC_SAW_SetAmp (OSC_SAW *osc, float amp)
{
  osc->amp = amp > 1.0f ? 1.0f : amp < 0.0f ? 0.0f : amp;
}

float OSC_SAW_CalcStep (OSC_SAW *osc)
{
  if (!(osc->wavetable)) { return 0.0f; }
  osc->phi += osc->phiinc;
  osc->phi -= (float) (uint16_t) osc->phi;
  return (osc->wavetable [((uint16_t) (osc->phi * 4096))]) * osc->amp;
}

void OSC_SAW_Calc (OSC_SAW *osc, float *out, int length)
{
  uint16_t  i;
  float     y;

  if (!(osc->wavetable)) 
  { 
    for (i = 0; i < length; i++)
    {
      out [i] = 0.0f;
    }
    return; 
  }
  for (i = 0; i < length; i++)
  {
    osc->phi += osc->phiinc;
    osc->phi -= (float) (uint16_t) osc->phi;
    y = osc->wavetable [((uint16_t) (osc->phi * 4096))];
    out [i] = y * osc->amp;
  }
}



//*******************************************************************************
//*
//*
//*
//*******************************************************************************


void RingMod_Init (RING_MOD *ringmod)
{
  memset (ringmod, 0, sizeof (RING_MOD));
}


void RingMod_SetFreq (RING_MOD *ringmod, float freq)
{
  ringmod->phiinc = (uint32_t) (((float) 0xffffffff) * freq / 44100.0f);
}


void RingMod_SetAmp (RING_MOD *ringmod, float amp)
{
  amp = (amp < 0.0f) ? 0.0f : (amp > 1.0f) ? 1.0f : amp;
  ringmod->amp = amp;
}


void RingMod_SetDryWet (RING_MOD *ringmod, float drywet)
{
  drywet = (drywet < 0.0f) ? 0.0f : (drywet > 1.0f) ? 1.0f : drywet;
  ringmod->dry = drywet;
}

void RingMod_Do (RING_MOD *ringmod, float *inbuffer, float *outbuffer, int length)
{
  uint16_t i;
  float x, y;

  for (i = 0; i < length; i++)
  {
    ringmod->phi += ringmod->phiinc;
    x = inbuffer [i];
    y = ringmod->amp * sintab [ringmod->phi >> 20];
    y = ((x * y) * ringmod->dry) + (x * (1.0f - ringmod->dry));
    outbuffer [i] = y;
  }
}





//*******************************************************************************
//*
//*
//*
//*******************************************************************************


void FLT_Init (FILTER *flt)
{
  memset (flt, 0, sizeof (FILTER));
  flt->data.typ = 0xff;
  flt->data.pitch = -1.0f;
  flt->data.res = 0.3141f;
}
/*
void FLT_Set  (FILTER *flt, uint8_t typ, float pitch, float res)
{
  uint16_t  idx;
  float     w0, sinw0, cosw0, alpha, a0, q;


  if ((flt->data.typ != typ) || (flt->data.pitch != pitch) || (flt->data.res != res))
  {
    if (flt->data.typ != typ)
    {
      memset (&(flt->taps [0]), 0, sizeof (float) * 4);
      flt->data.typ = typ;
      if (typ == 0)
      {
        flt->coeffs [0] = 1.0f;
        flt->coeffs [1] = 0.0f;
        flt->coeffs [2] = 0.0f;
        flt->coeffs [3] = 0.0f;
        flt->coeffs [4] = 0.0f;
        return;
      }

    }

    flt->data.pitch = pitch;
    flt->data.res   = res;


    q = (float) res;

    pitch = pitch > 119.99f ? 119.99f : pitch < 0.0f ? 0.0f : pitch;
    idx = (uint16_t) (pitch * 100.0f);
    w0  =  2.0f * 3.1415926f * (note_phis [idx % 1200] / (float) (1 << (9 - (idx / 1200))));

    sinw0   = (float) sin (w0);
    cosw0   = (float) cos (w0);
    alpha   = (float) sinw0 / (2.0f * q);
    a0      = (float) 1.0f / (1.0f + alpha);

    flt->coeffs [3] = -a0 * (-2 * cosw0);
    flt->coeffs [4] = -a0 * ( 1 - alpha);

    switch (typ)
    {
      //lowpass
      case 1 :
        flt->coeffs [0] =  a0 * ((1 - cosw0) / 2.0f);
        flt->coeffs [1] =  a0 * ( 1 - cosw0);
        flt->coeffs [2] =  flt->coeffs [0];

        break;

      // highpass
      case 2 :
        flt->coeffs [0] =  a0 * ( (1 + cosw0) / 2.0f);
        flt->coeffs [1] =  a0 * (-(1 + cosw0));
        flt->coeffs [2] =  flt->coeffs [0];
        break;

      // bandpass
      case 3 :
        flt->coeffs [0] =  a0 * (     alpha);
        flt->coeffs [1] =  0;
        flt->coeffs [2] =  -flt->coeffs [0];
        break;

      // notch
      case 4 :
        flt->coeffs [0] =  a0;
        flt->coeffs [1] =  a0 * (-2.0f * cosw0);
        flt->coeffs [2] =  a0;
        break;
    }
  }
}
*/

void FLT_Set  (FILTER *flt, uint8_t typ, float pitch, float res)
{
  //uint16_t  idx;
  float     x, a, k, kq, n, qr;


  if (flt->precount) { flt->precount--; return; }
  else { flt->precount = 4; }

  if ((flt->data.typ != typ) || (flt->data.pitch != pitch) || (flt->data.res != res))
  {
    flt->data.typ   = typ;
    flt->data.pitch = pitch;
    flt->data.res   = res;

    LIMIT_VALUE (typ,       0 ,    2   );
    LIMIT_VALUE (pitch,   0.0f, 119.99f);
    LIMIT_VALUE (res,    -1.0f,   0.99f);


    if (typ == 0)
    {
      flt->coeffs [0] = 1.0f;
      flt->coeffs [1] = 0.0f;
      flt->coeffs [2] = 0.0f;
      flt->coeffs [3] = 0.0f;
      flt->coeffs [4] = 0.0f;
      return;
    }


    

   // pitch   = pitch > 119.99f ? 119.99f : pitch < 0.0f ? 0.0f : pitch;
   // q       = res < -1.0f ? -1.0f : res > 1.0f ? 1.0f : res;

    //qr      = (float) sqrt (2) * pow (10, -res);

    /*idx     = (uint16_t) (pitch * 100.0f);
    w0      =  3.1415926f * (note_phis [idx % 1200] / (float) (1 << (9 - (idx / 1200))));

    k       = (float) tan (w0);*/


    k       = k_table [(uint16_t) (pitch * 100.0f)];
    kq      = k * k;
    qr      = q_table [(uint8_t) ((1.0f + res) * 100.0f)];
    //qr      = (float) sqrt (2 * pow (10.0f, -q));

    n       = 1.0f / (1.0f + qr * k + kq);

    a =  2.0f;
    x =  0.0f;
    if      (typ == 1) { x = kq * n;  a =  2.0f; }
    else if (typ == 2) { x = n;       a = -2.0f; }

    flt->coeffs [0] = x ;
    flt->coeffs [1] = a * x;
    flt->coeffs [2] = x;
    flt->coeffs [3] = (-2.0f * (kq - 1)        * n);
    flt->coeffs [4] = ((-(1.0f - qr * k + kq)) * n);
/*
    switch (typ)
    {
      case 1 :
        flt->coeffs [0] = (kq * n) ;
        flt->coeffs [1] = 2.0f *  flt->coeffs [0];
        flt->coeffs [2] =         flt->coeffs [0];
        break;

      case 2 :
        flt->coeffs [0] = (1.0f  * n)                ;
        flt->coeffs [1] = -2.0f * flt->coeffs [0];
        flt->coeffs [2] =         flt->coeffs [0];
        break;
    }*/

  }
}


void FLT_Calc (FILTER *flt, float *inbuf, float *outbuf, int length)
{
  uint16_t i;
  float x, y;

  for (i = 0; i < length; i++)
  {
    x = inbuf [i];

    y = flt->coeffs [0] * x             +
        flt->coeffs [1] * flt->taps [0] + flt->coeffs [2] * flt->taps [1] +
        flt->coeffs [3] * flt->taps [2] + flt->coeffs [4] * flt->taps [3];

    flt->taps [1] = flt->taps [0];
    flt->taps [3] = flt->taps [2];

    flt->taps [0] = x;
    flt->taps [2] = y;

    outbuf [i] = y;
  }
}




void FLT_CalcStep (FILTER *flt, float *inbuf, float *outbuf)
{
  float x, y;

  x = *inbuf;

  y = flt->coeffs [0] * x             +
      flt->coeffs [1] * flt->taps [0] + flt->coeffs [2] * flt->taps [1] +
      flt->coeffs [3] * flt->taps [2] + flt->coeffs [4] * flt->taps [3];

  flt->taps [1] = flt->taps [0];
  flt->taps [3] = flt->taps [2];

  flt->taps [0] = x;
  flt->taps [2] = y;

  *outbuf = y;

}


//*******************************************************************************
//*
//*
//*
//*******************************************************************************

void OVR_Calc (float ovr, float *inp, float *outp, int length)
{
  uint16_t i, j, k, f;
  float x, y, a, b;

  for (i = 0; i < length; i++)
  {
    x = inp [i];
    x = x > 1.0f ? 1.0f : x < -1.0f ? -1.0f : x;
    j = (uint16_t) (((x < 0.0f ? -x : x)) * 65535.0f);
    f = j & 63;
    j = j >> 6;
    k = j + 1 > 1023 ? 1023 : j + 1;
    a = log_tab [j]; b = log_tab [k];
    if (x >= 0.0f)
    {
      y = a * ((float) f / 63.0f) + b * (1 - ((float) f / 63.0f));
    }
    else
    {
      y = a * ((float) f / 63.0f) + b * (1 - ((float) f / 63.0f));
    }
    y = y * ovr + x * (1.0f - ovr);
    outp [i] = y;
  }
}




//*******************************************************************************
//*
//*
//*
//*******************************************************************************


void EQ_Set  (EQ_FILTER *eq, float pitch, float res, float gain)
{
  uint16_t  idx;
  float     w0, k, kq, v0, n, q;


  if ((eq->data.gain != gain) || (eq->data.pitch != pitch) || (eq->data.res != res))
  {

    eq->data.pitch = pitch;
    eq->data.res   = res;
    eq->data.gain  = gain;

    q       = (float) res;

    pitch   = pitch > 119.99f ? 119.99f : pitch < 0.0f ? 0.0f : pitch;
    idx     = (uint16_t) (pitch * 100.0f);
    w0      =  3.1415926f * (note_phis [idx % 1200] / (float) (1 << (9 - (idx / 1200))));

    gain    = gain > 36.0f ? 36.0f : gain < -36.0f ? -36.0f : gain;

    v0      = (float) pow (10.0f, gain / 20.0f);
    k       = (float) tan (w0);
    kq      = k * k;

    if (v0 > 0.0f)
    {
      n               = 1.0f + (1.0f / q) * k + kq;

      eq->coeffs [0]  =   (1.0f + (v0 / q)     * k + kq) / n;
      eq->coeffs [1]  =    2.0f * (kq - 1)               / n;
      eq->coeffs [2]  =   (1.0f - (v0 / q)     * k + kq) / n;
      eq->coeffs [3]  = - (2.0f * (kq - 1.0f))           / n;
      eq->coeffs [4]  = - (1.0f - (1.0f /q)    * k + kq) / n;
    }
    else if (v0 == 0.0f)
    {
      eq->coeffs [0]  = 1.0f;
      eq->coeffs [1]  = 0.0f;
      eq->coeffs [2]  = 0.0f;
      eq->coeffs [3]  = 0.0f;
      eq->coeffs [4]  = 0.0f;
    }
    else if (v0 < 0.0f)
    {
      v0              = 1.0f / v0;
      n               = 1.0f + (v0 / q) * k + kq;

      eq->coeffs [0]  =  (1.0f + (1.0f / q)  * k + kq) / n;
      eq->coeffs [1]  =   2.0f * (kq - 1.0f)           / n;
      eq->coeffs [2]  =  (1.0f - (1 / q)     * k + kq) / n;
      eq->coeffs [3]  = -(2.0f * (kq - 1.0f))          / n;
      eq->coeffs [4]  = -(1.0f - (v0 / q)    * k + kq) / n;
    }
  }
}


void EQ_Set2  (EQ_FILTER *eq, float pitch, float q, float gain)
{
  float     n0, n1, k, kq, v0, v1, q1, n, c1, c2;


  if ((eq->data.gain != gain) || (eq->data.pitch != pitch) || (eq->data.res != q))
  {
    if (gain == 0.0f)
    {
      eq->coeffs [0]  = 1.0f;
      eq->coeffs [1]  = 0.0f;
      eq->coeffs [2]  = 0.0f;
      eq->coeffs [3]  = 0.0f;
      eq->coeffs [4]  = 0.0f;
      return;
    }

    LIMIT_VALUE (gain,  -24.0f,  23.99f);
    LIMIT_VALUE (pitch,   0.0f, 119.99f);
    LIMIT_VALUE (q,       0.1f,   10.0f);

    eq->data.pitch = pitch;
    eq->data.res   = q;
    eq->data.gain  = gain;


    k       = k_table [(uint16_t) (pitch * 100.0f)];
    kq      = k * k;
    v0      = db_table [       (uint16_t) ((gain + 24.0f) * 100.0f)];
    v1      = db_table [4799 - (uint16_t) ((gain + 24.0f) * 100.0f)];
    q1      = (1.0f / q);

    if      (v0 > 1.0f)
    {
      c1 = v0 * q1;
      c2 = q1;
      n  = 1.0f / (1.0f +  q1 * k + kq);
    }
    else if (v0 < 1.0f)
    {
      c1 = q1;
      c2 = v1 * q1;
      n  = 1.0f / (1.0f + v1 * q1 * k + kq);
    }

    n0 = (c1 * k + kq) * n;
    n1 = (c2 * k + kq) * n;

    eq->coeffs [0]  =  n + n0;
    eq->coeffs [1]  =   2.0f * (kq - 1.0f)  * n;
    eq->coeffs [2]  =  n - n0;
    eq->coeffs [3]  = -eq->coeffs [1];
    eq->coeffs [4]  = - (n - n1) * n;


    /*
    if (v0 > 1.0f)
    {

      n               = 1.0f / (1.0f +  cn * k + kq);
      */
/*
      n               = 1.0f + (1.0f / q) * k + kq;
      eq->coeffs [0]  =   (1.0f + (v0 / q)     * k + kq) / n;
      eq->coeffs [1]  =    2.0f * (kq - 1)               / n;
      eq->coeffs [2]  =   (1.0f - (v0 / q)     * k + kq) / n;
      eq->coeffs [3]  = - (2.0f * (kq - 1.0f))           / n;
      eq->coeffs [4]  = - (1.0f - (1.0f /q)    * k + kq) / n;
 */
/*      eq->coeffs [0]  =   n + n0;
      eq->coeffs [1]  =    2.0f * (kq - 1.0f)  * n;
      eq->coeffs [2]  =   n - n0;
      eq->coeffs [3]  = - eq->coeffs [1];
      eq->coeffs [4]  = - (n - n0) * n;
    }
    else if (v0 < 1.0f)
    {
      n               = 1.0f / (1.0f + cn * k + kq);
*/
/*
      v0              = 1.0f / v0;
      n               = 1.0f + (v0 / q) * k + kq;
      eq->coeffs [0]  =  (1.0f + (1.0f / q)  * k + kq) / n;
      eq->coeffs [1]  =   2.0f * (kq - 1.0f)           / n;
      eq->coeffs [2]  =  (1.0f - (1 / q)     * k + kq) / n;
      eq->coeffs [3]  = -(2.0f * (kq - 1.0f))          / n;
      eq->coeffs [4]  = -(1.0f - (v0 / q)    * k + kq) / n;
 */
  /*    eq->coeffs [0]  =  n + n0;
      eq->coeffs [1]  =   2.0f * (kq - 1.0f)  * n;
      eq->coeffs [2]  =  n - n0;
      eq->coeffs [3]  = -eq->coeffs [1];
      eq->coeffs [4]  = - (n - n0) * n;
    }*/
  }
}


void EQ_Calc (EQ_FILTER *eq, float *inbuf, float *outbuf, int length)
{
  uint16_t i;
  float x, y;

  for (i = 0; i < length; i++)
  {
    x = inbuf [i];

    y = eq->coeffs [0] * x            +
        eq->coeffs [1] * eq->taps [0] + eq->coeffs [2] * eq->taps [1] +
        eq->coeffs [3] * eq->taps [2] + eq->coeffs [4] * eq->taps [3];

    eq->taps [1] = eq->taps [0];
    eq->taps [3] = eq->taps [2];

    eq->taps [0] = x;
    eq->taps [2] = y;

    outbuf [i] = y;
  }
}





//*******************************************************************************
//*
//*
//*
//*******************************************************************************



void PRESETS_Assign   (struct _PRESETS *prs, uint32_t numpars, void *data, uint32_t size, void *bank, uint32_t numpres, void *user)
{
  prs->data_sng   = data;
  prs->size       = size;
  prs->data_pbank = bank;
  prs->numpres    = numpres;
  prs->numpars    = numpars;
  prs->user       = user;

  prs->initfunc   = PRESETS_initfunc;
  prs->getfunc    = PRESETS_getfunc;
  prs->setfunc    = PRESETS_setfunc;
  prs->loadfunc   = PRESETS_loadfunc;
  prs->savefunc   = PRESETS_savefunc;
  prs->getfunc    = PRESETS_getfunc;
}


void PRESETS_initfunc   (struct _PRESETS *prs, void *data, void *user)
{
  if ((prs->data_sng) && (prs->size) && (prs->size <= 1024)) 
  {
    memset (prs->data_sng,    0, (uint32_t) prs->size);
  }

  if ((prs->data_pbank) && (prs->size) && (prs->size <= 1024) && (prs->numpres <= 128)) 
  {
    memset (prs->data_pbank,  0, (uint32_t) prs->size * (uint32_t) prs->numpres);
  }
}
/*
char ReadHex (FILE *file, uint8_t *val)
{
  uint8_t   d, n, c1, c2, f;

  if (fread (&c1, 1, 1, file) != 1) { return 0; } 
  if (fread (&c2, 1, 1, file) != 1) { return 0; } 

  f =       ((c1 >= '0') && (c1 <= '9')) ||
            ((c1 >= 'a') && (c1 <= 'f')) ||
            ((c1 >= 'A') && (c1 <= 'F'));
  f = f &&  ((c2 >= '0') && (c2 <= '9')) ||
            ((c2 >= 'a') && (c2 <= 'f')) ||
            ((c2 >= 'A') && (c2 <= 'F'));

  if (!f) { return 0; }

  d = 0;
  n = (c1 >= 'a') ? c1 + 10 - 'a' : (c1 >= 'A') ? c1 + 10 - 'A' : (c1 >= '0') ? c1 - '0' : 0;
  d |= n << 4;
  n = (c2 >= 'a') ? c2 + 10 - 'a' : (c2 >= 'A') ? c2 + 10 - 'A' : (c2 >= '0') ? c2 - '0' : 0;
  d |= n << 0;

  *val = d;
  return 1;
}
*/
void PRESETS_loadfunc   (struct _PRESETS *prs, void *storage)
{
/*  FILE     *file;
  uint32_t  i, j, v = 0, f = 0;
  uint16_t  s;
  uint8_t   n, t [10], *p;

  file = fopen ((char*) storage, "rb+");

  if (file)
  {
    f = 1;
    f &= ReadHex (file, &t [0]);
    f &= ReadHex (file, &t [1]);
    f &= ReadHex (file, &t [2]);
    f &= ReadHex (file, &t [3]);
    f &= ReadHex (file, &t [4]);
    f &= ReadHex (file, &t [5]);
    f &= ReadHex (file, &t [6]);

    v = ((uint32_t) t [0]) << 24 | 
        ((uint32_t) t [1]) << 16 | 
        ((uint32_t) t [2]) <<  8 | 
        ((uint32_t) t [3]) <<  0; 
    s = ((uint16_t) t [4]) <<  8 |
        ((uint16_t) t [5]) <<  0;
    n = ((uint8_t)  t [6]) <<  0;

    f = f & ((v == prs->id) && (s == prs->size) && (n == prs->numpres));

    if (f)
    {
      for (i = 0; i < ((uint32_t) prs->numpres + 1); i++)
      {
        for (j = 0; j < prs->size; j++)
        {   
          if (i == 0) { p = &(((uint8_t *) prs->data_sng)   [j]); }
          else        { p = &(((uint8_t *) prs->data_pbank) [(i - 1) * prs->size + j]); }
          f &= ReadHex (file, p);
        }
      }
    }
    fclose (file);
  }

  if (!f)
  {
    //prs->initfunc (prs,);
  }*/
}

void PRESETS_savefunc   (struct _PRESETS *prs, void *storage)
{
/*  FILE     *file;
  uint32_t  i, j;
  uint8_t   d;

  file = fopen ((char*) storage, "wb+");

  if (file)
  {
    fprintf (file, "%08x%04x%02x", prs->id, prs->size, prs->numpres);

    for (i = 0; i < ((uint32_t) prs->numpres + 1); i++)
    {
      for (j = 0; j < prs->size; j++)
      {   
        if (i == 0) { d = ((uint8_t *) prs->data_sng)   [j]; }
        else        { d = ((uint8_t *) prs->data_pbank) [(i - 1) * prs->size + j]; }
        fprintf (file, "%02x", d);
      }
    }
    fclose (file);
  }*/
}



void PRESETS_getfunc    (struct _PRESETS *prs, uint8_t prn, uint8_t idx, uint8_t *val)
{

}

void PRESETS_setfunc    (struct _PRESETS *prs, uint8_t prn, uint8_t idx, uint8_t val)
{

}

void PRESETS_touifunc   (struct _PRESETS *prs)
{

}


void PRESETS_Call (struct _PRESETS *prs, uint8_t prsnum)
{
  uint32_t i;
  uint8_t  v;

  if (prsnum < prs->numpres)
  {
    for (i = 0; i < prs->numpars; i++)
    {
      prs->getfunc (prs, prsnum, i, &v);
      prs->setfunc (prs, 0xff, i, v);
    }    
  }
}


void PRESETS_Store (struct _PRESETS *prs, uint8_t prsnum)
{
  uint32_t i;
  uint8_t  v;

  if (prsnum < prs->numpres)
  {
    for (i = 0; i < prs->numpars; i++)
    {
      prs->getfunc (prs, 0xff, i, &v);
      prs->setfunc (prs, prsnum, i, v);
    }    
  }
}

void PRESETS_Refresh (struct _PRESETS *prs)
{
  uint32_t i;
  uint8_t  v;

  for (i = 0; i < prs->numpars; i++)
  {
    prs->getfunc (prs, 0xff, i, &v);
    prs->setfunc (prs, 0xff, i, v);
  }    

}





//*******************************************************************************
//*
//*
//*
//*******************************************************************************


#define PI      (3.14159265358979f)



float Pitch2Freq (float note)
{
  uint16_t idx;
  float    frq;

  note  = note > 119.99f ? 119.99f : note < 0.0f ? 0.0f : note;
  idx   = (uint16_t) (note * 100.0f);
  frq   =  44100 * (note_phis [idx % 1200] / (float) (1 << (9 - (idx / 1200))));
  return frq;
}

void MFS_Init (MOOG_FILTER_STAGE *mfs, float newSampleRate)
{
  mfs->sampleRate = newSampleRate;
  mfs->z1 = 0;
}


void MFS_setFc(MOOG_FILTER_STAGE *mfs, float fc)
{
  // prewarp the cutoff- these are bilinear-transform filters
  float wd = 2.0f * PI * fc;
  float T = 1.0f / mfs->sampleRate;
  float wa = (2.0f / T) * (float) tan (wd * T / 2.0f);
  float g = wa * T / 2.0f;
  // calculate big G value; see Zavalishin p46 The Art of VA Design
  mfs->G = g / (1.0f + g);
}


// TPT 1 pole filter; see Zavalishin p46 The Art of VA Design
float MFS_doFilterStage(MOOG_FILTER_STAGE *mfs, float xn)
{
  float v = (xn - mfs->z1) * mfs->G;
  float out = v + mfs->z1;
  mfs->z1 = out + v;
  return out;
}



void MLF_Init (MOOG_LADDER_FILTER *mlf, float newSampleRate)
{
  MFS_Init (&(mlf->filter [0]), newSampleRate);
  MFS_Init (&(mlf->filter [1]), newSampleRate);
  MFS_Init (&(mlf->filter [2]), newSampleRate);
  MFS_Init (&(mlf->filter [3]), newSampleRate);
  MLF_calculateTPTCoeffs (mlf, 69.0f, 0.707f);
}

void MLF_calculateTPTCoeffs (MOOG_LADDER_FILTER *mlf, float cutoff, float Q)
{
  float f0 = Pitch2Freq (cutoff);
  // 4 sync-tuned filters
  MFS_setFc (&(mlf->filter [0]), f0);
  MFS_setFc (&(mlf->filter [1]), f0);
  MFS_setFc (&(mlf->filter [2]), f0);
  MFS_setFc (&(mlf->filter [3]), f0);

  // NOTE: Q is limited to 20 on the UI to prevent blowing up
  // Q = 0.707 -> 25 ==> k = 0->4
  mlf->k = 4.0f * (Q - 0.707f) / (25.0f - 0.707f);
  // ours
  mlf->fc = f0;
  mlf->pitch = cutoff;
  mlf->q = Q;
}

float MLF_doTPTMoogLPF (MOOG_LADDER_FILTER *mlf, float xn)
{
  // calculate g
  float wd  = 2 * PI * mlf->fc;
  float T   = 1.0f / (float) mlf->filter [0].sampleRate;
  float wa  = (2.0f / T) * (float) tan (wd * T / 2.0f);
  float g   = wa * T / 2.0f;
  float G   = g * g * g * g;
  float S   = g * g * g * mlf->filter [0].z1 +
                  g * g * mlf->filter [1].z1 +
                      g * mlf->filter [2].z1 +
                          mlf->filter [3].z1;

  // u is input to filters, straight from book
  float u = (xn - mlf->k * S) / (1.0f + mlf->k * G);
  // four cascades using nested functions
  float filterOut = MFS_doFilterStage (&(mlf->filter [3]), MFS_doFilterStage (&(mlf->filter [2]),
                    MFS_doFilterStage (&(mlf->filter [1]), MFS_doFilterStage (&(mlf->filter [0]), u))));
  // output
  return filterOut;
}



//*******************************************************************************
//*
//*
//*
//*******************************************************************************



void MoogLadder_Init (MOOG_LADDER *ml)
{
  memset (ml, 0, sizeof (ml));

  Moogladder_Set (ml, 81.0f, 0.707f);
}

void Moogladder_Set (MOOG_LADDER *ml, float cutoff, float res)
{
  float g;

  if (res != ml->res)
  {
    if (res < 0.0f)  { res = 0.0f;  }
    if (res > 0.82f) { res = 0.82f; }
    ml->k     = 4.0f * res;
    ml->res   = res;
  }

  if (ml->cut != cutoff)
  {
    g = (float) tan (PI * Pitch2Freq (cutoff) / (float) 44100.0f);

    ml->g     = g / (1.0f + g);
    ml->g1    = g;
    ml->g2    = ml->g1 * g;
    ml->g3    = ml->g2 * g;
    ml->g4    = ml->g3 * g;
    ml->cut   = cutoff;
  }
}

void MoogLadder_Calc (MOOG_LADDER *ml, float *in, float *out, int size)
{
  float v, o,  u,  s4;
  float g, g1, g2, g3, g4, k;

  g   = ml->g;
  g1  = ml->g1;
  g2  = ml->g2;
  g3  = ml->g3;
  g4  = ml->g4;
  k   = ml->k;

  while (size)
  {
    s4 = g3 * ml->z [0] + g2 * ml->z [1] + g1 * ml->z [2] + ml->z [3];

    u = (*in - k * s4) / (1.0f + k * g4);

    // four cascades using nested functions

    v = (u - ml->z [0]) * g;
    o = v + ml->z [0];
    ml->z [0] = o + v;

    v = (o - ml->z [1]) * g;
    o = v + ml->z [1];
    ml->z [1] = o + v;

    v = (o - ml->z [2]) * g;
    o = v + ml->z [2];
    ml->z [2] = o + v;

    v = (o - ml->z [3]) * g;
    o = v + ml->z [3];
    ml->z [3] = o + v;

    *out = o;

    out++;
    in++;
    size--;
  }
}


#ifdef CPP


class CTPTMoogFilterStage
{
  public:
    CTPTMoogFilterStage(void);
    ~CTPTMoogFilterStage(void);

  protected:
    // controls
    float G; // cutoff
    float scalar;  // scalar value
    float sampleRate; // fs
    float z1;// z-1 storage location

  public:
    inline void initialize(float newSampleRate)
    {
      // save
      sampleRate = newSampleRate;
      z1 = 0;
    }
    void setFc(float fc);
    float doFilterStage(float xn);
    float getSampleRate(){return sampleRate;}
    float getStorageRegisterValue(){return z1;}
};



void CTPTMoogFilterStage::setFc(float fc)
{
  // prewarp the cutoff- these are bilinear-transform filters
  float wd = 2*pi*fc;
  float T = 1/sampleRate;
  float wa = (2/T)*tan(wd*T/2);
  float g = wa*T/2;
  // calculate big G value; see Zavalishin p46 The Art of VA Design
  G = g/(1.0 + g);
}



// TPT 1 pole filter; see Zavalishin p46 The Art of VA Design
float CTPTMoogFilterStage::doFilterStage(float xn)
{
  float v = (xn - z1)*G;
  float out = v + z1;
  z1 = out + v;
  return out;
}



class CTPTMoogLadderFilter
{
  public:
    CTPTMoogLadderFilter(void);
    ~CTPTMoogLadderFilter(void);

  protected:
    CTPTMoogFilterStage filter1;
    CTPTMoogFilterStage filter2;
    CTPTMoogFilterStage filter3;
    CTPTMoogFilterStage filter4;
    float k; // Q control
    float fc; // fc control

  public:
    inline void initialize(float newSampleRate)
    {
      filter1.initialize(newSampleRate);
      filter2.initialize(newSampleRate);
      filter3.initialize(newSampleRate);
      filter4.initialize(newSampleRate);
    }
    inline void calculateTPTCoeffs(float cutoff, float Q)
    {
      // 4 sync-tuned filters
      filter1.setFc(cutoff);
      filter2.setFc(cutoff);
      filter3.setFc(cutoff);
      filter4.setFc(cutoff);

      // NOTE: Q is limited to 20 on the UI to prevent blowing up
      // Q = 0.707 -> 25 ==> k = 0->4
      k = 4.0*(Q - 0.707)/(25.0 - 0.707);
      // ours
      fc = cutoff;
    }
    float doTPTMoogLPF(float xn);
};


float CTPTMoogLadderFilter::doTPTMoogLPF(float xn)
{
  // calculate g
  float wd = 2*pi*fc;
  float T = 1/(float)filter1.getSampleRate();
  float wa = (2/T)*tan(wd*T/2);
  float g = wa*T/2;
  float G = g*g*g*g;
  float S = g*g*g*filter1.getStorageRegisterValue() +
      g*g*filter2.getStorageRegisterValue() +
      g*filter3.getStorageRegisterValue() +
      filter4.getStorageRegisterValue();
  // u is input to filters, straight from book
  float u = (xn - k*S)/(1 + k*G);
  // four cascades using nested functions
  float filterOut = filter4.doFilterStage(filter3.doFilterStage
        (filter2.doFilterStage(filter1.doFilterStage(u))));
  // output
  return filterOut;
}

#endif
