

  #ifndef AUDIO_BASICS_H
    #define AUDIO_BASICS_H


      //*******************************************************************************
      //*
      //*
      //*
      //*******************************************************************************


      #define LIMIT_VALUE(x,a,b)    x = ((x) > (b) ? (b) : ( (x) < (a) ? (a) : (x)))


      //*******************************************************************************
      //*
      //*
      //*
      //*******************************************************************************

      typedef struct
      {
        uint8_t     at;
        uint8_t     al;
        int8_t      ac;
        uint8_t     dt;
        int8_t      dc;
        uint8_t     sl;
        uint8_t     rt;
        int8_t      rc;
      } ENV_AHDSR_DATA;

      typedef struct
      {
        ENV_AHDSR_DATA  env_data;

        float       att_phiinc;
        float       att_curve;
        float       att_level;
        float       dec_phiinc;
        float       dec_curve;
        float       sus_level;
        float       rel_phiinc;
        float       rel_curve;  

        float       phase_level;
        float       phase_delta;
        float       phase_phi;  
        float       curr_level;
        float       curr_gate;
        float       curr_curve;
        float       last_gate;
        float       gate_int;
        uint8_t     phase;
      } ENV_AHDSR;



      void      ENV_Init        (ENV_AHDSR *env);
      void      ENV_Set         (ENV_AHDSR *env, uint8_t at, uint8_t al, int8_t ac, uint8_t dt, int8_t dc, uint8_t sl, uint8_t rt, int8_t rc);
      void      ENV_SetItem     (ENV_AHDSR *env, uint8_t item, uint8_t val);
      void      ENV_SetStruct   (ENV_AHDSR *env, ENV_AHDSR_DATA *data);
      void      ENV_SetCalc     (ENV_AHDSR *env);
      float     ENV_Step        (ENV_AHDSR *env, uint8_t gate);




      //*******************************************************************************
      //*
      //*
      //*
      //*******************************************************************************

      typedef struct
      {
        float note;
        float phi;
        float phiinc;  
        float amp;
      } OSC_SIN;


      void  OSC_SIN_Init      (OSC_SIN *osc);
      void  OSC_SIN_SetNote   (OSC_SIN *osc, float note);
      void  OSC_SIN_SetAmp    (OSC_SIN *osc, float amp);
      float OSC_SIN_CalcStep  (OSC_SIN *osc);
      void  OSC_SIN_Calc      (OSC_SIN *osc, float *out, int length);




      //*******************************************************************************
      //*
      //*
      //*
      //*******************************************************************************

      typedef struct
      {
        float   note;
        float   phi;
        float   phiinc;  
        float   amp;
        float  *wavetable;
        uint8_t tab;
      } OSC_SAW;


      void  OSC_SAW_Init      (OSC_SAW *osc);
      void  OSC_SAW_SetNote   (OSC_SAW *osc, float note);
      void  OSC_SAW_SetAmp    (OSC_SAW *osc, float amp);
      float OSC_SAW_CalcStep  (OSC_SAW *osc);
      void  OSC_SAW_Calc      (OSC_SAW *osc, float *out, int length);





      //*******************************************************************************
      //*
      //*
      //*
      //*******************************************************************************


      typedef struct
      {
        uint32_t  phi;
        uint32_t  phiinc;
        float     freq;
        float     amp;
        float     dry;
      } RING_MOD;


      void RingMod_Init       (RING_MOD *ringmod);
      void RingMod_SetFreq    (RING_MOD *ringmod, float freq);
      void RingMod_SetAmp     (RING_MOD *ringmod, float amp);
      void RingMod_SetDryWet  (RING_MOD *ringmod, float drywet);
      void RingMod_Calc       (RING_MOD *ringmod, float *inbuffer, float *outbuffer, int length);



      //*******************************************************************************
      //*
      //*
      //*
      //*******************************************************************************



      typedef struct
      {
        uint8_t typ;
        float   pitch;
        float   res;
      } FILTER_DATA;


      typedef struct
      {
        FILTER_DATA data;

        float coeffs [5];
        float taps   [4];

        uint8_t precount;
      } FILTER;



      void FLT_Init     (FILTER *flt);
      void FLT_Set      (FILTER *flt, uint8_t typ, float pitch, float res);
      void FLT_Calc     (FILTER *flt, float *inbuf, float *outbuf, int length);
      void FLT_CalcStep (FILTER *flt, float *inbuf, float *outbuf);



      void OVR_Calc     (float ovr, float *inp, float *outp, int length);




      //*******************************************************************************
      //*
      //*
      //*
      //*******************************************************************************

      typedef struct
      {
        float   gain;
        float   pitch;
        float   res;
      } EQ_DATA;

      typedef struct
      {
        EQ_DATA data;

        float coeffs [5];
        float taps   [4];

      } EQ_FILTER;




      void EQ_Set  (EQ_FILTER *flt, float pitch, float res, float gain);
      void EQ_Calc (EQ_FILTER *flt, float *inbuf, float *outbuf, int length);



      //*******************************************************************************
      //*
      //*
      //*
      //*******************************************************************************

      struct _PRESETS;
      
      typedef void    (* PRS_INIT)   (struct _PRESETS *prs, void *data, void *user);
      typedef void    (* PRS_LOAD)   (struct _PRESETS *prs, void *storage);
      typedef void    (* PRS_SAVE)   (struct _PRESETS *prs, void *storage);
      typedef void    (* PRS_GET)    (struct _PRESETS *prs, uint8_t prn, uint8_t idx, uint8_t *val);
      typedef void    (* PRS_SET)    (struct _PRESETS *prs, uint8_t prn, uint8_t idx, uint8_t  val);
      typedef void    (* PRS_TO_UI)  (struct _PRESETS *prs);


      typedef struct _PRESETS
      {
        void       *data_sng;
        void       *data_pbank;
        void       *user;
        uint32_t    id;
        uint8_t     size;
        uint8_t     numpars;
        uint8_t     numpres;

        PRS_INIT    initfunc;
        PRS_LOAD    loadfunc;
        PRS_SAVE    savefunc;
        PRS_GET     getfunc;
        PRS_SET     setfunc;
        PRS_TO_UI   touifunc;
      } PRESETS;



      void PRESETS_Assign     (struct _PRESETS *prs, uint32_t numpars, void *data, uint32_t size, void *bank, uint32_t numprs, void *user);
      void PRESETS_Call       (struct _PRESETS *prs, uint8_t prsnum);
      void PRESETS_Store      (struct _PRESETS *prs, uint8_t prsnum);
      void PRESETS_Refresh    (struct _PRESETS *prs);

      void PRESETS_initfunc   (struct _PRESETS *prs, void *data, void *user);
      void PRESETS_loadfunc   (struct _PRESETS *prs, void *storage);
      void PRESETS_savefunc   (struct _PRESETS *prs, void *storage);
      void PRESETS_getfunc    (struct _PRESETS *prs, uint8_t prn, uint8_t idx, uint8_t *val);
      void PRESETS_setfunc    (struct _PRESETS *prs, uint8_t prn, uint8_t idx, uint8_t  val);
      void PRESETS_touifunc   (struct _PRESETS *prs);


      //*******************************************************************************
      //*
      //*
      //*
      //*******************************************************************************


      typedef struct
      {
        float G; // cutoff
        //float scalar;  // scalar value
        float sampleRate; // fs
        float z1;// z-1 storage location
      } MOOG_FILTER_STAGE;

      typedef struct
      {
        MOOG_FILTER_STAGE filter [4];
        float k; // Q control
        float fc; // fc control
        float q;
        float pitch;
      } MOOG_LADDER_FILTER;


      void  MLF_Init                (MOOG_LADDER_FILTER *mlf, float newSampleRate);
      void  MLF_calculateTPTCoeffs  (MOOG_LADDER_FILTER *mlf, float cutoff, float Q);
      float MLF_doTPTMoogLPF        (MOOG_LADDER_FILTER *mlf, float xn);





      typedef struct
      {
        float g;
        float g1;
        float g2;
        float g3;
        float g4;
        float z [4];
        float k;
        float cut;
        float res;
      } MOOG_LADDER;

      void MoogLadder_Init  (MOOG_LADDER *ml);
      void Moogladder_Set   (MOOG_LADDER *ml, float cutoff, float Q);
      void MoogLadder_Calc  (MOOG_LADDER *ml, float *in, float *out, int size);



      //*******************************************************************************
      //*
      //*
      //*
      //*******************************************************************************


      void OSCSAWTableInit (void);

    #endif
