////////////////////////////////////////////////////////////////////////////
//
//  codec.c
//
//	Initialisiert und Konfiguriert den Audio-Codec (TLV320AIC23B)
//  Std. mssig luft der Codec mit 48Khz@20Bit
//  Gesteuert werden kann das Routing, die Lautstrke (Line & HP) und Mute
//  SPI wird SW mssig emuliert.
//
//	Author 	: Rene Bllhoff
//  Created : 
//
//	(c) 2005-2008 by Rene Bllhoff - reneboellhoff (at) gmx (dot) de - 
//
//////////////////////////////////////////////////////////////////////////////
//
//  This source file may be used and distributed without
//  restriction provided that this copyright statement is not
//  removed from the file and that any derivative work contains
//  the original copyright notice and the associated disclaimer.
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//										
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public
//  License along with this source; if not, download it	from
//  http://www.gnu.org/licenses/gpl.html	
//
////////////////////////////////////////////////////////////////////////////	

  #include "stdio.h"
	#include "stdint.h"

  #include "stm32f4xx_hal.h"

  #include "codec.h"





#define vCodecClkLo()     HAL_GPIO_WritePin (TLV_CLK_PORT,  TLV_CLK_BIT,  GPIO_PIN_RESET);
#define vCodecClkHi()     HAL_GPIO_WritePin (TLV_CLK_PORT,  TLV_CLK_BIT,  GPIO_PIN_SET);
#define vCodecDataLo()    HAL_GPIO_WritePin (TLV_DOUT_PORT, TLV_DOUT_BIT, GPIO_PIN_RESET);
#define vCodecDataHi()    HAL_GPIO_WritePin (TLV_DOUT_PORT, TLV_DOUT_BIT, GPIO_PIN_SET);


#define vCodecSelect()    HAL_GPIO_WritePin (TLV_CS1_PORT, TLV_CS1_BIT,   GPIO_PIN_RESET);
#define vCodecDeselect()  HAL_GPIO_WritePin (TLV_CS1_PORT, TLV_CS1_BIT,   GPIO_PIN_SET);


#define vTLVDelayPin()    vCodecWriteDelay (1000)



uint8_t ucVolInL;
uint8_t ucVolInR;
uint8_t  bInSel;
uint8_t  bDACOn;
uint8_t  bBypass;

uint8_t ucLineInLevel = 0x17;


void vCodecRouteReg (void);


void vCodecInitPort (void)
{

  vCodecDeselect ();
  vCodecClkHi    ();
  vCodecDataHi   ();
}

void vCodecWriteDelay (uint16_t uiData)
{
  volatile uint16_t i;

  for (i = 0; i < uiData; i++)
  {
    asm ("nop");
  }
}

void vCodecWriteWord (uint16_t uiData)
{
  uint8_t ucC;
  
  vCodecSelect ();
	vTLVDelayPin ();
  for (ucC = 0 ; ucC < 16; ucC ++)
  {
    vCodecClkLo (); 
    if (uiData & 0x8000) { vCodecDataHi (); }
                    else { vCodecDataLo (); }
		vTLVDelayPin ();
    vCodecClkHi (); 
		vTLVDelayPin ();
    uiData = uiData << 1;
  }
	vTLVDelayPin ();
  vCodecDeselect ();
  vTLVDelayPin ();
}

void vCodecWriteRegister (uint8_t ucReg, uint16_t uiData)
{
  ucReg   &= 0x0F;
  uiData  &= 0x01FF;
  
  uiData = (((uint16_t) (ucReg)) << 9) | uiData;
  vCodecWriteWord (uiData);
}





void vCodecHPVol 	(uint8_t ucLevel)
{
	ucLevel &= 0x3f;
	ucLevel = ucLevel == 0 ? 0x30 : ucLevel + 0x40;

  vCodecWriteRegister (0x02, 0x0180 + ucLevel);
//	vCodecWriteRegister (0x03, 0x0180 + ucLevel);
}


void vCodecLineIn	(uint8_t ucLevel)
{
  if (ucLevel > 0x1f) { ucLevel = 0x1f; }
  vCodecWriteRegister (0x00, 0x0100 + ucLevel);
	ucLineInLevel = ucLevel;
}

uint8_t   ucCodecGetLineIn (void)
{
  return ucLineInLevel;
}

void vCodecInput		(uint8_t bMicOrLine)
{
	bInSel = bMicOrLine;
	vCodecRouteReg();
}


void vCodecMute		(uint8_t bMute)
{
	bDACOn = !bMute;
	vCodecRouteReg();
}


void vCodecBypass (uint8_t bByp)
{
	bBypass = bByp;
	vCodecRouteReg();
}



void vCodecRouteReg (void)
{
  uint8_t ucTmp;

	ucTmp = ((bDACOn   ? 1 : 0) << 4) | // ausgang (dac) an
					((bInSel   ? 1 : 0) << 2) | // line in oder mic
					((bInSel   ? 0 : 1) << 1) | // mic mute (wenn line in)
					((bBypass  ? 1 : 0) << 3);  // bypass
	vCodecWriteRegister (0x04, ucTmp);
}

void vCodecInit (void)
{
  vCodecInitPort();

  HAL_Delay(10);

  // codec resetten
  vCodecWriteRegister (0x0f, 0x0000);


  HAL_Delay(1);
  
  vCodecWriteRegister (0x00, 0x0117);		// lautstrke l std
  vCodecWriteRegister (0x01, 0x0117);		// lautstrke r std
  vCodecWriteRegister (0x02, 0x01d5); 	// kopfh. lautstrke l std
  vCodecWriteRegister (0x03, 0x01d5); 	// kopfh. lautstrke l std
  vCodecWriteRegister (0x04, 0x0012);		//
  vCodecWriteRegister (0x05, 0x0000); 	//
  vCodecWriteRegister (0x06, 0x0000); 	//
  vCodecWriteRegister (0x07, 0x0042); 	//
  vCodecWriteRegister (0x08, 0x0000); 	//
  vCodecWriteRegister (0x09, 0x0001); 	//


	bDACOn  = 1;
	bBypass = 0;
	bInSel  = 0;
}



#ifdef PARSER_EXEC_FUNC

	PARSER_EXEC_FUNC (vCodecCmd)
	{
	  GET_PAR_U8 (ucPar1, 0);

		switch (ucCmd)
		{
		  case CODEC_BYP_ON 	: vCodecBypass 	(1);   break;
		  case CODEC_BYP_OFF 	: vCodecBypass 	(0);  break;
		  case CODEC_MUTEON 	: vCodecMute   	(1);   break;
		  case CODEC_MUTEOFF 	: vCodecMute   	(0);  break;
			case CODEC_LINEIN 	: vCodecLineIn 	(ucPar1); break;
			case CODEC_LINESEL 	: vCodecInput	 	(ucPar1); break;
			case CODEC_HPVOL	 	: vCodecHPVol 	(ucPar1); break;
		}
	}

#endif

