////////////////////////////////////////////////////////////////////////////
//
//  uParse_cfg.h
//
//  Konfiguration des uParsers
//   - Grundeinstellungen (z.B. benutzte Teile)
//   - Funktionen (Aufrufe und Codecs)
//   - Befehlstexte
//
//  Abschnitte
//   - UPARSE_CONFIG          - Grundeinstellungen
//   - UPARSE_FUNCTION        - Aufzurufende Funktionen
//   - UPARSE_USER_DATA_TYPE  - Weitere Datentypen in Parameter-Element mit aufnehmen
//   - UPARSE_USER_CODEC      - Benutzer-Definierte Codecs (z.b. Bool, Datum/Zeit)
//   - UPARSE_ITEM            - Befehle die der Parser erkennen kann
//
//  Author  : Rene Bllhoff
//  Created : around Apr 10
//
//  (c) 2005-2010 by Rene Bllhoff - reneboellhoff (at) gmx (dot) de - 
//
//////////////////////////////////////////////////////////////////////////////
//
//  This source file may be used and distributed without
//  restriction provided that this copyright statement is not
//  removed from the file and that any derivative work contains
//  the original copyright notice and the associated disclaimer.
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//                    
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public
//  License along with this source; if not, download it from
//  http://www.gnu.org/licenses/gpl.html  
//
////////////////////////////////////////////////////////////////////////////  

  // Grundeinstellungen bzw Grundkonfiguration
  #ifdef UPARSE_CONFIG

    // Erstellen der Parameterdatenstruktur
    #define PARSE_DATA_ELEMENTS         6     // max. Anzahl der Datenelemente
    #define PARSE_DATA_STR_LEN          80    // max. Lnge der Namen/Strings

    // Benutze automatisch generierte Hilfe
    #define PARSE_USES_HELP
    // Zeichenausgabefunktion fr Hilfe
    #define PARSE_PRINT_HELP_CHAR(x)        user_tx_put (x)
    
    // Benutze User-Definierte Codecs
    #define PARSE_USES_CODECS
    // Ignoriere Gro/Kleinschreibung
    #define PARSE_IGNORE_CASE


    // wichtige Includes (fr printf und Datentypen)

    #include <stdio.h>
    #include <string.h>

    #include "usb_uart_fifo.h"

  #endif

  // Hier sind alle aufzurufenden Funktionen aufgelistet.
  // Diese werden bei func unter UPARSE_ITEM eingetragen.
  #ifdef UPARSE_FUNCTION
  //UPARSE_FUNCTION (name,                  function        ) 
    UPARSE_FUNCTION (STD_CMD,               vStdCmd         ) 
  #endif

  // Hier werden weitere Datentypen an die Parameter-Element-Union "gehngt"
  #ifdef UPARSE_USER_DATA_TYPE
    float rReal;
  #endif

  // Hier werden eigene Codecs eingetragen
  #ifdef UPARSE_USER_CODEC
  //UPARSE_USER_CODEC (name,                  function        , displayname)
  //UPARSE_USER_CODEC (CODEC_STRING2,         cCodecString2   , "string")

  #endif

  // In den UPARSE_ITEM-Texten gibt es folgende steuerzeichen :
  // \x01 = zahl (z.b. -1, 0x34, -0b1100110, $1234, ...)
  // \x02 = name erstes zeichen alpha + '_', danach mit zahlen (z.b. _XY945, TEXT01, ...) 
  // \x03 = string (kompletten 8 bit) (also "hallo welt")
  // alles ab \x04 -> eigene Codecs

  #ifdef UPARSE_ITEM
  //UPARSE_ITEM(name,             text,                                 func    )


    UPARSE_ITEM(LISTCMD1,         "test",                               STD_CMD)

  #endif
