/*
 * usb_uart_fifo.c
 *
 *  Created on: 11.05.2017
 *      Author: TheMason
 */

#include <stdio.h>
#include <stdint.h>
#include <stdarg.h>
#include <string.h>

#include "stm32f4xx_hal.h"

#include "usb_uart_fifo.h"

#include "uParse.h"



/*
 *
 * UART1 = Raspberry PI
 * UART2 = Bluetooth Audio (optional)
 * UART4 = User
 * UART5 = Aux
 *
 */

/*
UART_FIFO usb_rx_fifo;
UART_FIFO usb_tx_fifo;
*/

UART_FIFO rpi_rx_fifo;
UART_FIFO rpi_tx_fifo;

UART_FIFO bta_rx_fifo;
UART_FIFO bta_tx_fifo;

UART_FIFO user_rx_fifo;
UART_FIFO user_tx_fifo;

UART_FIFO aux_rx_fifo;
UART_FIFO aux_tx_fifo;



uint8_t uart1_receive, uart2_receive, uart4_receive, uart5_receive;

uint8_t uart1_transmit, uart2_transmit, uart4_transmit, uart5_transmit;


extern UART_HandleTypeDef huart1;
extern UART_HandleTypeDef huart2;
extern UART_HandleTypeDef huart4;
extern UART_HandleTypeDef huart5;


void UART_FIFO_init (UART_FIFO *fifo)
{
  memset (fifo->data, 0, sizeof (fifo->data));
  fifo->wp = 0;
  fifo->rp = 0;
}

uint8_t UART_FIFO_put (UART_FIFO *fifo, uint8_t x)
{
  uint16_t i;

  i = (fifo->wp + 1) & (UART_FIFO_SIZE - 1);

  if (i != fifo->rp)
  {
    fifo->data [fifo->wp] = x;
    fifo->wp = i;
    return 1;
  }
  return 0;
}

uint8_t UART_FIFO_get (UART_FIFO *fifo, uint8_t *x)
{
  if (fifo->wp != fifo->rp)
  {
    *x = fifo->data [fifo->rp];
    fifo->rp = (fifo->rp + 1) & (UART_FIFO_SIZE - 1);
    return 1;
  }
  return 0;
}

uint8_t UART_FIFO_empty (UART_FIFO *fifo)
{
  return (fifo->wp == fifo->rp) ? 1 : 0;
}

uint8_t UART_FIFO_bytes (UART_FIFO *fifo)
{
  return (fifo->wp - fifo->rp) & (UART_FIFO_SIZE - 1);
}



/*
void usb_rx_tx_init (void)
{
  UART_FIFO_init (&usb_rx_fifo);
  UART_FIFO_init (&usb_tx_fifo);
}

uint8_t usb_rx_put (uint8_t x)
{
  return UART_FIFO_put (&usb_rx_fifo, x);
}

uint8_t usb_rx_get (uint8_t *x)
{
  return UART_FIFO_get (&usb_rx_fifo, x);
}

uint8_t usb_tx_put (uint8_t x)
{
  return UART_FIFO_put (&usb_tx_fifo, x);
}

uint8_t usb_tx_get (uint8_t *x)
{
  return UART_FIFO_get (&usb_tx_fifo, x);
}

uint8_t usb_tx_bytes (void)
{
  return UART_FIFO_bytes (&usb_tx_fifo);
}

uint8_t usb_rx_bytes (void)
{
  return UART_FIFO_bytes (&usb_rx_fifo);
}

void usb_tx_printf (char *fmt, ...)
{
  va_list args;
  char buffer [1024];
  uint16_t x, l;

  va_start (args, fmt);
  vsprintf (buffer, fmt, args);
  va_end (args);

  l = strlen (buffer);
  for (x = 0; x < l; x++)
  {
    usb_tx_put (buffer [x]);
  }

}
*/








void rpi_rx_tx_init (void)
{
  UART_FIFO_init (&rpi_rx_fifo);
  UART_FIFO_init (&rpi_tx_fifo);
}

uint8_t rpi_rx_put (uint8_t x)
{
  return UART_FIFO_put (&rpi_rx_fifo, x);
}

uint8_t rpi_rx_get (uint8_t *x)
{
  return UART_FIFO_get (&rpi_rx_fifo, x);
}

uint8_t rpi_tx_put (uint8_t x)
{
  return UART_FIFO_put (&rpi_tx_fifo, x);
}

uint8_t rpi_tx_get (uint8_t *x)
{
  return UART_FIFO_get (&rpi_tx_fifo, x);
}

uint8_t rpi_tx_bytes (void)
{
  return UART_FIFO_bytes (&rpi_tx_fifo);
}

uint8_t rpi_rx_bytes (void)
{
  return UART_FIFO_bytes (&rpi_rx_fifo);
}

void rpi_tx_printf (char *fmt, ...)
{
  va_list args;
  char buffer [1024];
  uint16_t x, l;

  va_start (args, fmt);
  vsprintf (buffer, fmt, args);
  va_end (args);

  l = strlen (buffer);
  for (x = 0; x < l; x++)
  {
    rpi_tx_put (buffer [x]);
  }

}



/*
void bta_rx_tx_init (void)
{
  UART_FIFO_init (&bta_rx_fifo);
  UART_FIFO_init (&bta_tx_fifo);
}

uint8_t bta_rx_put (uint8_t x)
{
  return UART_FIFO_put (&bta_rx_fifo, x);
}

uint8_t bta_rx_get (uint8_t *x)
{
  return UART_FIFO_get (&bta_rx_fifo, x);
}

uint8_t bta_tx_put (uint8_t x)
{
  return UART_FIFO_put (&bta_tx_fifo, x);
}

uint8_t bta_tx_get (uint8_t *x)
{
  return UART_FIFO_get (&bta_tx_fifo, x);
}

uint8_t bta_tx_bytes (void)
{
  return UART_FIFO_bytes (&bta_tx_fifo);
}

uint8_t bta_rx_bytes (void)
{
  return UART_FIFO_bytes (&bta_rx_fifo);
}


void bta_tx_printf (char *fmt, ...)
{
  va_list args;
  char buffer [1024];
  uint16_t x, l;

  va_start (args, fmt);
  vsprintf (buffer, fmt, args);
  va_end (args);

  l = strlen (buffer);
  for (x = 0; x < l; x++)
  {
    bta_tx_put (buffer [x]);
  }

}

*/





void aux_rx_tx_init (void)
{
  UART_FIFO_init (&aux_rx_fifo);
  UART_FIFO_init (&aux_tx_fifo);
}

uint8_t aux_rx_put (uint8_t x)
{
  return UART_FIFO_put (&aux_rx_fifo, x);
}

uint8_t aux_rx_get (uint8_t *x)
{
  return UART_FIFO_get (&aux_rx_fifo, x);
}

uint8_t aux_tx_put (uint8_t x)
{
  return UART_FIFO_put (&aux_tx_fifo, x);
}

uint8_t aux_tx_get (uint8_t *x)
{
  return UART_FIFO_get (&aux_tx_fifo, x);
}

uint8_t aux_tx_bytes (void)
{
  return UART_FIFO_bytes (&aux_tx_fifo);
}

uint8_t aux_rx_bytes (void)
{
  return UART_FIFO_bytes (&aux_rx_fifo);
}


void aux_tx_printf (char *fmt, ...)
{
  va_list args;
  char buffer [1024];
  uint16_t x, l;

  va_start (args, fmt);
  vsprintf (buffer, fmt, args);
  va_end (args);

  l = strlen (buffer);
  for (x = 0; x < l; x++)
  {
    aux_tx_put (buffer [x]);
  }

}





void user_rx_tx_init (void)
{
  UART_FIFO_init (&user_rx_fifo);
  UART_FIFO_init (&user_tx_fifo);
}

uint8_t user_rx_put (uint8_t x)
{
  return UART_FIFO_put (&user_rx_fifo, x);
}

uint8_t user_rx_get (uint8_t *x)
{
  return UART_FIFO_get (&user_rx_fifo, x);
}

uint8_t user_tx_put (uint8_t x)
{
  return UART_FIFO_put (&user_tx_fifo, x);
}

uint8_t user_tx_get (uint8_t *x)
{
  return UART_FIFO_get (&user_tx_fifo, x);
}

uint8_t user_tx_bytes (void)
{
  return UART_FIFO_bytes (&user_tx_fifo);
}

uint8_t user_rx_bytes (void)
{
  return UART_FIFO_bytes (&user_rx_fifo);
}


void user_tx_printf (char *fmt, ...)
{
  va_list args;
  char buffer [1024];
  uint16_t x, l;

  va_start (args, fmt);
  vsprintf (buffer, fmt, args);
  va_end (args);

  l = strlen (buffer);
  for (x = 0; x < l; x++)
  {
    user_tx_put (buffer [x]);
  }

}




void InitUARTS (void)
{
  user_rx_tx_init();
  rpi_rx_tx_init();
  //bta_rx_tx_init();
  aux_rx_tx_init();

  StartUARTReceive ();
}




void HandleShellComInput (void)
{
  static uint8_t input_line [81] = { 0, };
  static uint8_t input_ptr;
  uint8_t ch;

  if (SHELL_GETCH (&ch))
  {
    switch (ch)
    {
      case 0x7f :
      case 0x08 :
        if (input_ptr > 0)
        {
          SHELL_PUTSTR ("\x08 \x08");
          input_line [input_ptr--] = 0;
          input_line [input_ptr  ] = 0;
        }
        break;

      case 0x0d :
        SHELL_PUTSTR ("\n\r");
        if (cParserProcess ((int8_t *) input_line) == 0)
        {
          SHELL_PUTSTR ("\r\nOk\r\n");
        }
        else
        {
          SHELL_PUTSTR ("\r\nError\r\n");
        }
        memset (input_line, 0, sizeof (input_line));
        input_ptr = 0;
        break;

      case 0x1b :
        break;

      default :
        if (input_ptr < 80)
        {
          SHELL_PUTCH (ch);
          input_line [input_ptr++] = ch;
        }
        break;
    }
  }
}




void HAL_UART_RxCpltCallback (UART_HandleTypeDef *huart)
{
  if      (huart == &huart1)
  {
    HAL_UART_Receive_IT (&huart1, &uart1_receive, 1);
    rpi_rx_put (uart1_receive);
  }
  else if (huart == &huart2)
  {
    HAL_UART_Receive_IT (&huart2, &uart2_receive, 1);
    //bta_rx_put(uart2_receive);
  }
  else if (huart == &huart4)
  {
    HAL_UART_Receive_IT (&huart4, &uart4_receive, 1);
    user_rx_put (uart4_receive);
  }
  else if (huart == &huart5)
  {
    HAL_UART_Receive_IT (&huart5, &uart5_receive, 1);
    aux_rx_put (uart5_receive);
  }
}


void HAL_UART_TxCpltCallback (UART_HandleTypeDef *huart)
{
/*  if ((huart == &huart1) && (rpi_tx_get (&uart1_transmit)))
  {
    HAL_UART_Transmit_IT(&huart1, &uart1_transmit, 1);
  }
  else if ((huart == &huart3) && (bta_tx_get (&uart3_transmit)))
  {
    HAL_UART_Transmit_IT(&huart3, &uart3_transmit, 1);
  }*/
}


void StartUARTReceive (void)
{
  HAL_UART_Receive_IT (&huart1, &uart1_receive, 1);
 // HAL_UART_Receive_IT (&huart2, &uart2_receive, 1);
  HAL_UART_Receive_IT (&huart4, &uart4_receive, 1);
  HAL_UART_Receive_IT (&huart5, &uart5_receive, 1);
}

void Handle_UARTs (void)
{
  uint8_t state;

  state = HAL_UART_GetState(&huart1);

  if ((state == HAL_UART_STATE_READY) || (state == HAL_UART_STATE_BUSY_RX))
  {
    if (rpi_tx_get (&uart1_transmit))
    {
      HAL_UART_Transmit_IT (&huart1, &uart1_transmit, 1);
    }
  }

/*
  state = HAL_UART_GetState(&huart2);

  if ((state == HAL_UART_STATE_READY) || (state == HAL_UART_STATE_BUSY_RX))
  {
    if (rpi_tx_get (&uart2_transmit))
    {
      HAL_UART_Transmit_IT (&huart2, &uart2_transmit, 1);
    }
  }
*/
  state = HAL_UART_GetState(&huart4);

  if ((state == HAL_UART_STATE_READY) || (state == HAL_UART_STATE_BUSY_RX))
  {
    if (user_tx_get (&uart4_transmit))
    {
      HAL_UART_Transmit_IT (&huart4, &uart4_transmit, 1);
    }
  }

  state = HAL_UART_GetState(&huart5);

  if ((state == HAL_UART_STATE_READY) || (state == HAL_UART_STATE_BUSY_RX))
  {
    if (aux_tx_get (&uart5_transmit))
    {
      HAL_UART_Transmit_IT (&huart5, &uart5_transmit, 1);
    }
  }
}


